
module Main where

-- friends
import Random.Array

import qualified Solver.Accelerate      as A
import qualified Solver.SDK             as S

import Data.Array.Accelerate            as A
import Data.Array.Accelerate.IO         as A
import Data.Array.Accelerate.CUDA       as A

-- standard library
import Prelude                          as P
import Criterion.Main
import System.Random.MWC
import Foreign.CUDA                     as CUDA
import qualified Data.Vector.Storable   as V


doTest :: Int -> IO ()
doTest x =
  let n = x * 1000000   -- million

      copyToDevice vec =
        V.unsafeWith vec $ \ptr -> do
          dev <- CUDA.mallocArray n
          CUDA.pokeArray n ptr dev
          return dev

  in do
    -- generate some random data
    --
    psy         <- randomArrayIO (\_ -> uniformR ((5,1,0.25), (30,100,10))) (Z:.n)
    let ((((),sp_vec), os_vec), oy_vec) = toVectors psy

    sp_dev      <- copyToDevice sp_vec
    os_dev      <- copyToDevice os_vec
    oy_dev      <- copyToDevice oy_vec

    -- Run the benchmark
    --
    defaultMain
      [ bgroup ("n=" ++ shows x "M")
          [ bench "sdk"         $ S.blackscholes n sp_dev os_dev oy_dev
          , bench "accelerate"  $ whnf (run1 A.blackscholes) psy
          ]
      ]

    -- Cleanup
    --
    CUDA.free sp_dev
    CUDA.free os_dev
    CUDA.free oy_dev


main :: IO ()
main = mapM_ doTest [2,4..20]

